@@ -1,7 +1,12 @@
-/* $Id: email_conduit.c,v 1.18 2003/01/26 22:29:20 toshok Exp $ */
+/* $Id$ */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <glib.h>
-#include <gnome.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
 #include <pi-source.h>
 #include <pi-socket.h>
@@ -32,54 +37,100 @@
 #define LOG(format,args...)
 #endif
 
+#define EC_PAD 8
+
+const char *EC_SEND_ACTION [] = {"delete", "file"};
+const char *EC_RECV_ACTION [] = {"copy", "delete", "mirror"};
+
 GnomePilotConduit *conduit_get_gpilot_conduit( guint32 pilotId ) ;
 void conduit_destroy_gpilot_conduit( GnomePilotConduit *c );
+static gboolean save_config (GKeyFile    *kfile, const gchar *conf);
+static GKeyFile* get_kfile (const gchar *conf);
+static void migrate_conf (const gchar *old, const gchar *new);
+extern time_t parsedate(char *p);
+
 
 static void 
 load_configuration(ConduitCfg **c,guint32 pilotId) 
 {
-	gchar *prefix;
+	gchar *iPilot;
+	GKeyFile *kfile;
+	GError *error = NULL;
 
 	g_assert(c!=NULL);
 	*c = g_new0(ConduitCfg,1);
 	(*c)->child = -1;
 
-	prefix = g_strdup_printf("/gnome-pilot.d/email-conduit/Pilot_%u/",pilotId);
-  
-	gnome_config_push_prefix(prefix);
-	(*c)->sendmail = gnome_config_get_string( "sendmail=/usr/lib/sendmail -t -i");
-	(*c)->fromAddr = gnome_config_get_string( "from_address" );
-	(*c)->sendAction = gnome_config_get_string( "send_action=file");
-	(*c)->mhDirectory = gnome_config_get_string( "mh_directory" );
-	(*c)->mboxFile = gnome_config_get_string ( "mbox_file" );
-	(*c)->receiveAction = gnome_config_get_string( "receive_action=copy" );
-	gnome_config_pop_prefix();
+ 	kfile = get_kfile("email-conduit");
+	iPilot = g_strdup_printf ("Pilot_%u", pilotId);
+
+	(*c)->sendmail = g_key_file_get_string (kfile, iPilot, "sendmail", &error);
+	if (error) {
+	    g_warning (_("Unable load key email-conduit/%s/sendmail: %s"), iPilot, error->message);
+	    g_error_free (error);
+	    error = NULL;
+	    (*c)->sendmail = g_strdup ("/usr/lib/sendmail -t -i");
+	}
+	(*c)->fromAddr = g_key_file_get_string (kfile, iPilot, "from_address", NULL);
+	(*c)->sendAction = g_key_file_get_string (kfile, iPilot, "send_action", &error);
+	if (error) {
+	    g_warning (_("Unable load key email-conduit/%s/send_action: %s"), iPilot,
+		error->message);
+	    g_error_free (error);
+	    error = NULL;
+	    (*c)->sendAction = g_strdup ("file");
+	}
+	(*c)->mhDirectory = g_key_file_get_string (kfile, iPilot, "mh_directory", NULL);
+	(*c)->mboxFile = g_key_file_get_string (kfile, iPilot, "mbox_file", NULL);
+	(*c)->receiveAction = g_key_file_get_string (kfile, iPilot,
+	    "receive_action", &error);
+	if (error) {
+	    g_warning (_("Unable load key email-conduit/%s/receive_action: %s"), iPilot,
+		error->message);
+	    g_error_free (error);
+	    error = NULL;
+	    (*c)->receiveAction = g_strdup ("copy");
+	}
+
 
 	(*c)->pilotId = pilotId;
-	g_free(prefix);
+
+	g_free (iPilot);
+	g_key_file_free (kfile);
 }
 
 static void
 save_configuration(ConduitCfg *c)
 {
-	gchar *prefix;
+	gchar *iPilot;
+	GKeyFile *kfile;
 
 	g_assert(c!=NULL);
 
-	prefix = g_strdup_printf("/gnome-pilot.d/email-conduit/Pilot_%u/",c->pilotId);
+ 	kfile = get_kfile("email-conduit");
+	iPilot = g_strdup_printf ("Pilot_%u", c->pilotId);
 
-	gnome_config_push_prefix(prefix);
-	gnome_config_set_string("sendmail", c->sendmail);
-	gnome_config_set_string("from_address", c->fromAddr);
-	gnome_config_set_string("send_action", c->sendAction);
-	gnome_config_set_string("mh_directory", c->mhDirectory);
-	gnome_config_set_string("mbox_file", c->mboxFile);
-	gnome_config_set_string("receive_action", c->receiveAction);
-	gnome_config_pop_prefix();
-	gnome_config_sync();
-	gnome_config_drop_all();
+	if (c->sendmail != NULL)
+	    g_key_file_set_string (kfile, iPilot, "sendmail", c->sendmail);
+	else
+	    g_key_file_remove_key (kfile, iPilot, "sendmail", NULL);
+	if (c->fromAddr != NULL)
+	    g_key_file_set_string (kfile, iPilot, "from_address", c->fromAddr);
+	else
+	    g_key_file_remove_key (kfile, iPilot, "from_addres", NULL);
+	g_key_file_set_string (kfile, iPilot, "send_action", c->sendAction);
+	if (c->mhDirectory != NULL)
+	    g_key_file_set_string (kfile, iPilot, "mh_directory", c->mhDirectory);
+	else
+	    g_key_file_remove_key (kfile, iPilot, "mh_directory", NULL);
+	if (c->mboxFile != NULL)
+	    g_key_file_set_string (kfile, iPilot, "mbox_file", c->mboxFile);
+	else
+	    g_key_file_remove_key (kfile, iPilot, "mbox_file", NULL);
+	g_key_file_set_string (kfile, iPilot, "receive_action", c->receiveAction);
 
-	g_free(prefix);
+	g_free(iPilot);
+	save_config (kfile, "email-conduit");
 }
 
 static void 
@@ -132,7 +183,7 @@ destroy_configuration(ConduitCfg **c)
 	*c = NULL;
 }
 
-void markline( char *msg ) 
+void markline( unsigned char *msg ) 
 {
     while( (*msg) != '\n' && (*msg) != 0 ) {
         msg++; 
@@ -221,9 +272,9 @@ static gint match_mail(gconstpointer a, gconstpointer b)
 
 static gboolean
 write_message_to_pilot (GnomePilotConduit *c, GnomePilotDBInfo *dbi, 
-			int dbHandle, char *buffer, int msg_num)
+			int dbHandle, unsigned char *buffer, int msg_num)
 {
-    char *msg;
+    unsigned char *msg;
     int h;
     struct Mail t;
     int len;
@@ -241,7 +292,15 @@ write_message_to_pilot (GnomePilotConduit *c, GnomePilotDBInfo *dbi,
     t.body = NULL;
     t.dated = 0;
 
-    msg = (char *)buffer;
+    /* initialise these to something */
+    t.read = 0;
+    t.signature = 0;
+    t.confirmRead = 0;
+    t.confirmDelivery = 0;
+    t.priority = 0;
+    t.addressing = 0;
+
+    msg = buffer;
     h = 1;
     
     while ( h == 1 ) {
@@ -255,9 +314,9 @@ write_message_to_pilot (GnomePilotConduit *c, GnomePilotDBInfo *dbi,
 	    h = 0;
 	    header( &t, 0 );
 	} else {
-	    header( &t, msg );
+	    header( &t, (char *) msg );
 	}
-	msg += strlen(msg)+1;
+	msg += strlen((char *)msg)+1;
     }
     
     if ( (*msg) == 0 ) {
@@ -270,19 +329,19 @@ write_message_to_pilot (GnomePilotConduit *c, GnomePilotDBInfo *dbi,
 	return FALSE;
     }
     
-    t.body = strdup( msg );
+    t.body = strdup( (char *) msg );
     
     len = pack_Mail( &t, buffer, 0xffff );
 
     /* if this mail already exists in the Palms inbox then skip this mail */
     needle.size = len;
     needle.buffer = buffer;
-    inbox_list = (GList*) gtk_object_get_data(GTK_OBJECT(c), "inbox_list");
+    inbox_list = (GList*) g_object_get_data(G_OBJECT(c), "inbox_list");
     field = g_list_find_custom(inbox_list, &needle, match_mail);
     if (field) {
     	/* remove the mail from the list as we've already seen it */
 	inbox_list = g_list_remove_link(inbox_list, field);
-	gtk_object_set_data(GTK_OBJECT(c),"inbox_list",(gpointer)inbox_list);
+	g_object_set_data(G_OBJECT(c),"inbox_list",(gpointer)inbox_list);
 	free(field->data);
 	g_list_free_1(field);
 	LOG("Skipping message (already on Palm device)");
@@ -301,7 +360,11 @@ write_message_to_pilot (GnomePilotConduit *c, GnomePilotDBInfo *dbi,
 static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi ) 
 {
     int dbHandle;
+#ifdef PILOT_LINK_0_12
+    pi_buffer_t *pi_buf;
+#else
     guchar buffer[0xffff];
+#endif
     struct MailAppInfo tai;
     struct MailSyncPref pref;
     struct MailSignaturePref sig;
@@ -319,9 +382,15 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
         fprintf( stderr, "Unable to open mail database\n" );
         return( -1 );
     }
-    
+
+#ifdef PILOT_LINK_0_12
+    pi_buf = pi_buffer_new (0xffff);    
+    dlp_ReadAppBlock( dbi->pilot_socket, dbHandle, 0, 0xffff, pi_buf);
+    unpack_MailAppInfo( &tai, pi_buf->data, 0xffff );
+#else
     dlp_ReadAppBlock( dbi->pilot_socket, dbHandle, 0, buffer, 0xffff );
     unpack_MailAppInfo( &tai, buffer, 0xffff );
+#endif
    
     pref.syncType = 0;
     pref.getHigh = 0;
@@ -331,6 +400,26 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
     pref.filterFrom = 0;
     pref.filterSubject = 0;
 
+#ifdef PILOT_LINK_0_12
+     if ( pi_version( dbi->pilot_socket ) > 0x0100 ) {
+         if ( dlp_ReadAppPreference( dbi->pilot_socket, makelong("mail"), 1, 1, 
+                                    pi_buf->allocated, pi_buf->data, 0, 0 ) >= 0 ) {
+            unpack_MailSyncPref( &pref, pi_buf->data, pi_buf->allocated );
+         } else { 
+             if ( dlp_ReadAppPreference( dbi->pilot_socket, makelong("mail"), 1,
+                                        1, pi_buf->allocated, pi_buf->data, 0, 0 ) >= 0 ) { 
+                unpack_MailSyncPref( &pref, pi_buf->data, pi_buf->allocated); 
+             } else {
+ 	      LOG("Couldn't get any mail preferences.\n",0);
+             }
+         } 
+ 
+          if ( dlp_ReadAppPreference( dbi->pilot_socket, makelong("mail"), 3, 1, 
+                                    pi_buf->allocated, pi_buf->data, 0, 0 ) > 0 ) {
+            unpack_MailSignaturePref( &sig, pi_buf->data, pi_buf->allocated);
+         } 
+     }
+#else
     if ( pi_version( dbi->pilot_socket ) > 0x0100 ) {
         if ( dlp_ReadAppPreference( dbi->pilot_socket, makelong("mail"), 1, 1, 
                                     0xffff, buffer, 0, 0 ) >= 0 ) {
@@ -349,18 +438,25 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
             unpack_MailSignaturePref( &sig, buffer, 0xffff );
         } 
     }
+#endif
 
     for ( i = 0; ; i++ ) {
         struct Mail t;
         int attr;
-        int size;
+#ifndef PILOT_LINK_0_12
+        int size=0;
+#endif
         recordid_t recID;
         int length;
         FILE * sendf;
-        
+
+#ifdef PILOT_LINK_0_12
+         length = dlp_ReadNextRecInCategory( dbi->pilot_socket, dbHandle, 1,
+                                            pi_buf, &recID, 0, &attr );	
+#else
         length = dlp_ReadNextRecInCategory( dbi->pilot_socket, dbHandle, 1,
-                                            buffer, &recID, 0, &size, &attr );
-        
+                                            buffer, &recID, 0, &size, &attr );	
+#endif
         if ( length < 0 ) {
             break;
         }
@@ -368,9 +464,13 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
         if ( ( attr & dlpRecAttrDeleted ) || ( attr & dlpRecAttrArchived ) ) {
             continue;
         }
-        
+
+#ifdef PILOT_LINK_0_12
+	unpack_Mail( &t, pi_buf->data, pi_buf->used);
+#else
         unpack_Mail( &t, buffer, length );
-       
+#endif
+ 
         sendf = popen( GET_CONFIG(c)->sendmail, "w" );
         if ( sendf == NULL ) {
             fprintf( stderr, "Unable to create sendmail process\n" );
@@ -442,6 +542,7 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
         }
 
         if ( pclose( sendf ) != 0 ) {
+            free_Mail( &t );
             fprintf( stderr, "Error ending sendmail exchange\n" );
             continue;
         }
@@ -450,8 +551,13 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
             dlp_DeleteRecord( dbi->pilot_socket, dbHandle, 0, recID );
         } else if ( !strcmp( GET_CONFIG(c)->sendAction, 
                              "file" ) ) {
+#ifdef PILOT_LINK_0_12
+             dlp_WriteRecord( dbi->pilot_socket, dbHandle, attr, recID, 3, 
+                             pi_buf->data, pi_buf->used, 0);
+#else
             dlp_WriteRecord( dbi->pilot_socket, dbHandle, attr, recID, 3, 
                              buffer, size, 0);
+#endif
         }
         free_Mail( &t );
     }
@@ -459,17 +565,29 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
     /* read in all the existing records on the Palm so that we can
      * spot duplicate mails
      */
-    inbox_list = (GList*) gtk_object_get_data(GTK_OBJECT(c), "inbox_list");
+    inbox_list = (GList*) g_object_get_data(G_OBJECT(c), "inbox_list");
     if ( strcmp( GET_CONFIG(c)->receiveAction, "copy" ) == 0 ||
          strcmp( GET_CONFIG(c)->receiveAction, "mirror" ) == 0 ) {
     	for ( i = 0; ; i++ ) {
-	    int attr, length, size;
+	    int attr, length;
+#ifndef PILOT_LINK_0_12
+	    int size;
+#endif
 	    recordid_t recID;
 	    MailDBRecord *record;
 
 	    /* iterate through records in category 0 (Inbox) ... */
+#ifdef PILOT_LINK_0_12
+ 	    length = dlp_ReadNextRecInCategory( dbi->pilot_socket, dbHandle, 0,
+						pi_buf, &recID, 0, &attr);
+	    /* pi-dlp.h does not state that the return value is the length...
+	     * so it seems safer to read the length from the pi_buf */
+	    if (length >= 0)
+		    length = pi_buf->used;
+#else
 	    length = dlp_ReadNextRecInCategory( dbi->pilot_socket, dbHandle, 0,
 						buffer, &recID, 0, &size, &attr);
+#endif
 	    if ( length < 0 ) {
 	    	break;
 	    }
@@ -479,7 +597,11 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
 	    record->recID = recID;
 	    record->size = length;
 	    record->buffer = ((guchar *) record) + sizeof(*record);
+#ifdef PILOT_LINK_0_12
+	    memcpy(record->buffer, pi_buf->data, length);
+#else
 	    memcpy(record->buffer, buffer, length);
+#endif
 	    inbox_list = g_list_append(inbox_list, record);
 	    LOG("storing record %d", recID);
 	}
@@ -488,7 +610,7 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
     /* the above loop is likely to change the value of inbox_list so we
      * must put it back
      */
-    gtk_object_set_data(GTK_OBJECT(c),"inbox_list",(gpointer)inbox_list);
+    g_object_set_data(G_OBJECT(c),"inbox_list",(gpointer)inbox_list);
 
 
 
@@ -523,19 +645,32 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
 #endif
             
             len = 0;
+#ifdef PILOT_LINK_0_12
+            while ( ( len < pi_buf->allocated ) &&
+                    ( ( l = read( mhmsg, (char *)(pi_buf->data+len),
+                                  pi_buf->allocated-len ) ) > 0 ) ) {
+                 len += l;
+             }
+            pi_buf->data[len] = 0;
+#else
             while ( ( len < sizeof(buffer) ) &&
                     ( ( l = read( mhmsg, (char *)(buffer+len),
                                   sizeof(buffer)-len ) ) > 0 ) ) {
                 len += l;
             }
             buffer[len] = 0;
-            
+#endif
+      
             if ( l < 0 ) {
                 fprintf( stderr, "Error processing message %d\n", i );
                 break;
             } 
-		
+
+#ifdef PILOT_LINK_0_12
+	    if (write_message_to_pilot (c, dbi, dbHandle, pi_buf->data, i)) {
+#else
 	    if (write_message_to_pilot (c, dbi, dbHandle, buffer, i)) {
+#endif
 		rec++;
                 if ( strcmp( GET_CONFIG(c)->receiveAction, "delete" ) == 0 ) {
                     char filename[1000];
@@ -562,36 +697,62 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
         f = fopen (GET_CONFIG (c)->mboxFile, "r");
 	
 
-	if (f) {	
-	    fgets (buffer, sizeof (buffer) - 1, f);
-	    while (!feof (f) && strncmp (buffer, "From ", 5)) {
-		fgets (buffer, sizeof (buffer) - 1, f);
+	if (f) {
+#ifdef PILOT_LINK_0_12
+	    if (fgets ((char *) pi_buf->data, pi_buf->allocated - 1, f) != NULL) {
+		while (!feof (f) && strncmp ((char *)pi_buf->data, "From ", 5)) {
+		    if (fgets ((char *)pi_buf->data, pi_buf->allocated - 1, f) == NULL)
+			break;
+		}
 	    }
+#else
+	    if (fgets (buffer, sizeof (buffer) - 1, f) != NULL) {
+		while (!feof (f) && strncmp (buffer, "From ", 5)) {
+		    if (fgets (buffer, sizeof (buffer) - 1, f) == NULL)
+			break;
+		}
+	    }
+#endif
+
 	    for( i = 1; !feof (f); i++ ) {
 		int len;
 		char *p;
            
 		LOG( "Processing message %d", i );
 		len = 0;
+#ifdef PILOT_LINK_0_12
+		while ( ( len < pi_buf->allocated ) &&
+			( ( p = fgets ( (char *)(pi_buf->data+len),
+					pi_buf->allocated-len, f ) ) != 0 ) ) {		
+#else
 		while ( ( len < sizeof(buffer) ) &&
 			( ( p = fgets ( (char *)(buffer+len),
 					sizeof(buffer)-len, f ) ) != 0 ) ) {
+#endif
 		    if (!strncmp (p, "From ", 5)) {
 			break;
 		    } else {
 			len += strlen (p);
 		    }
 		}
-		
+
+#ifdef PILOT_LINK_0_12
+		pi_buf->data[len] = 0;
+#else
 		buffer[len] = 0;
+#endif
 		len = 0;
 		
 		if ( len < 0 ) {
 		    fprintf( stderr, "Error processing message %d\n", i );
 		    break;
 		}
-		
+
+#ifdef PILOT_LINK_0_12
+		write_message_to_pilot (c, dbi, dbHandle, pi_buf->data, i);
+#else
 		write_message_to_pilot (c, dbi, dbHandle, buffer, i);
+#endif
 	    }
 	    fclose (f);
 	    if ( strcmp( GET_CONFIG(c)->receiveAction, "delete" ) == 0 ) {
@@ -608,7 +769,7 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
      * remaining on the inbox_list
      */
     if ( strcmp( GET_CONFIG(c)->receiveAction, "mirror" ) == 0 ) {
-	GList *elem = (GList*) gtk_object_get_data(GTK_OBJECT(c), "inbox_list");
+	GList *elem = (GList*) g_object_get_data(G_OBJECT(c), "inbox_list");
 	for (; elem != NULL; elem = elem->next) {
 	    MailDBRecord *record = (MailDBRecord *) elem->data;
 	    LOG("purging out of date record %d", record->recID);
@@ -621,6 +782,10 @@ static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
     dlp_ResetLastSyncPC( dbi->pilot_socket );
     dlp_CloseDB( dbi->pilot_socket, dbHandle );
 
+#ifdef PILOT_LINK_0_12
+    pi_buffer_free (pi_buf);
+#endif
+
     return( 0 );
 }
 
@@ -628,121 +793,75 @@ gint copy_from_pilot( GnomePilotConduit *c, GnomePilotDBInfo *dbi ) {
     return( synchronize( c, dbi ) );
 }
 
-void
-handleFileSelector (GtkWidget *widget, gpointer data)
-{
-    GtkWidget *fs = data;
-    GtkWidget *entry = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(fs), "entry"));
-    const gchar *fname;
-
-    fname = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fs));
-    gtk_entry_set_text(GTK_ENTRY(entry), fname);
-}
-
-
-void
-createFileSelector (GtkWidget *widget, gpointer data)
-{
-    GtkWidget *fs;
-
-    fs = gtk_file_selection_new(_("Select an mbox file or an MH directory"));
-    gtk_object_set_data(GTK_OBJECT(fs), "entry", (gpointer) data);
-    gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(fs)->ok_button),
-		"clicked", GTK_SIGNAL_FUNC(handleFileSelector), fs);
-
-    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fs)->ok_button),
-		"clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) fs);
-    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fs)->cancel_button),
-		"clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) fs);
-
-    gtk_widget_show (fs);
-    gtk_grab_add (fs); /* take focus from the settings dialog */
-}
-
 static GtkWidget
 *createCfgWindow (GnomePilotConduit* conduit)
 {
     GtkWidget *vbox, *table;
     GtkWidget *label, *widget;
-    GtkWidget *menu, *menuItem;
-    GtkWidget *box, *button;
 
-    vbox = gtk_vbox_new(FALSE, GNOME_PAD);
+    vbox = gtk_vbox_new(FALSE, EC_PAD);
 
     table = gtk_table_new(2, 5, FALSE);
     gtk_table_set_row_spacings(GTK_TABLE(table), 4);
     gtk_table_set_col_spacings(GTK_TABLE(table), 10);
-    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, GNOME_PAD);
+    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, EC_PAD);
 
     /* send_action option menu */
-    label = gtk_label_new(_("Send Action"));
-    widget = gtk_option_menu_new ();
-    menu = gtk_menu_new();
-    menuItem = gtk_menu_item_new_with_label (_("Delete from PDA"));
-    gtk_widget_show(menuItem);
-    gtk_object_set_data(GTK_OBJECT(menuItem), "short", "delete");
-    gtk_object_set_data(GTK_OBJECT(widget), "delete", (gpointer) 0);
-    gtk_menu_append (GTK_MENU (menu), GTK_WIDGET (menuItem));
-    menuItem = gtk_menu_item_new_with_label (_("File on PDA"));
-    gtk_widget_show(menuItem);
-    gtk_object_set_data(GTK_OBJECT(menuItem), "short", "file");
-    gtk_object_set_data(GTK_OBJECT(widget), "file", (gpointer) 1);
-    gtk_menu_append (GTK_MENU (menu), GTK_WIDGET (menuItem));
-    gtk_option_menu_set_menu (GTK_OPTION_MENU (widget), GTK_WIDGET (menu));
+    label = gtk_label_new(_("Send Action:"));
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    widget = gtk_combo_box_new_text ();
+    gtk_combo_box_append_text(GTK_COMBO_BOX(widget),  (_("Delete from PDA")));
+    g_object_set_data(G_OBJECT(widget), EC_SEND_ACTION[0], GINT_TO_POINTER(0));
+    gtk_combo_box_append_text(GTK_COMBO_BOX(widget), _("File on PDA"));
+    g_object_set_data(G_OBJECT(widget), EC_SEND_ACTION[1], GINT_TO_POINTER(1));
     gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
     gtk_table_attach_defaults(GTK_TABLE(table), widget, 1, 2, 0, 1);
-    gtk_object_set_data(GTK_OBJECT(vbox), "send_action", widget);
+    g_object_set_data(G_OBJECT(vbox), "send_action", widget);
 
     /* from_address entry */
     label = gtk_label_new(_("From:"));
-    widget = gtk_entry_new_with_max_length(128);
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    widget = gtk_entry_new();
+    gtk_entry_set_max_length(GTK_ENTRY(widget), 128);
     gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
     gtk_table_attach_defaults(GTK_TABLE(table), widget, 1, 2, 1, 2);
-    gtk_object_set_data(GTK_OBJECT(vbox), "from_address", widget);
+    g_object_set_data(G_OBJECT(vbox), "from_address", widget);
 
     /* sendmail entry */
-    label = gtk_label_new(_("Sendmail command"));
-    widget = gtk_entry_new_with_max_length(128);
+    label = gtk_label_new(_("Sendmail command:"));
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    widget = gtk_entry_new();
+    gtk_entry_set_max_length(GTK_ENTRY(widget), 128);
+
     gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
     gtk_table_attach_defaults(GTK_TABLE(table), widget, 1, 2, 2, 3);
-    gtk_object_set_data(GTK_OBJECT(vbox), "sendmail", widget);
+    g_object_set_data(G_OBJECT(vbox), "sendmail", widget);
 
     /* receive_action option menu */
-    label = gtk_label_new (_("Receive Action"));
-    widget = gtk_option_menu_new ();
-    menu = gtk_menu_new ();
-    menuItem = gtk_menu_item_new_with_label (_("Copy from Inbox"));
-    gtk_widget_show(menuItem);
-    gtk_object_set_data(GTK_OBJECT(menuItem), "short", "copy");
-    gtk_object_set_data(GTK_OBJECT(widget), "copy", (gpointer) 0);
-    gtk_menu_append (GTK_MENU (menu), GTK_WIDGET (menuItem));
-    menuItem = gtk_menu_item_new_with_label (_("Delete from Inbox"));
-    gtk_widget_show(menuItem);
-    gtk_object_set_data(GTK_OBJECT(menuItem), "short", "delete");
-    gtk_object_set_data(GTK_OBJECT(widget), "delete", (gpointer) 1);
-    gtk_menu_append (GTK_MENU (menu), GTK_WIDGET (menuItem));
-    menuItem = gtk_menu_item_new_with_label (_("Mirror Inbox"));
-    gtk_widget_show(menuItem);
-    gtk_object_set_data(GTK_OBJECT(menuItem), "short", "mirror");
-    gtk_object_set_data(GTK_OBJECT(widget), "mirror", (gpointer) 2);
-    gtk_menu_append (GTK_MENU (menu), GTK_WIDGET (menuItem));
-    gtk_option_menu_set_menu (GTK_OPTION_MENU (widget), GTK_WIDGET (menu));
+    label = gtk_label_new (_("Receive Action:"));
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    widget = gtk_combo_box_new_text ();
+    gtk_combo_box_append_text(GTK_COMBO_BOX(widget), _("Copy from Inbox"));
+    g_object_set_data(G_OBJECT(widget), EC_RECV_ACTION[0], GINT_TO_POINTER(0));
+    gtk_combo_box_append_text(GTK_COMBO_BOX(widget), _("Delete from Inbox"));
+    g_object_set_data(G_OBJECT(widget), EC_RECV_ACTION[1], GINT_TO_POINTER(1));
+    gtk_combo_box_append_text(GTK_COMBO_BOX(widget), _("Mirror Inbox"));
+    g_object_set_data(G_OBJECT(widget), EC_RECV_ACTION[2], GINT_TO_POINTER(2));
     gtk_table_attach_defaults (GTK_TABLE(table), label, 0, 1, 3, 4);
     gtk_table_attach_defaults(GTK_TABLE(table), widget, 1, 2, 3, 4);
-    gtk_object_set_data(GTK_OBJECT(vbox), "receive_action", widget);
+    g_object_set_data(G_OBJECT(vbox), "receive_action", widget);
+
 
     /* mbox_file entry */
-    label = gtk_label_new(_("Copy mail from"));
-    widget = gtk_entry_new_with_max_length(128);
-    button = gtk_button_new_with_label("...");
-    gtk_signal_connect(GTK_OBJECT(button), "clicked", 
-    	GTK_SIGNAL_FUNC(createFileSelector), widget);
-    box = gtk_hbox_new(FALSE, 0);
-    gtk_box_pack_end(GTK_BOX(box), button, FALSE, FALSE, 0);
-    gtk_box_pack_end(GTK_BOX(box), widget, TRUE, TRUE, 0);
+    label = gtk_label_new(_("Copy mail from:"));
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+
+    widget = gtk_file_chooser_button_new (_("Select a file"),
+	GTK_FILE_CHOOSER_ACTION_OPEN);
+
     gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 4, 5);
-    gtk_table_attach_defaults(GTK_TABLE(table), box, 1, 2, 4, 5);
-    gtk_object_set_data(GTK_OBJECT(vbox), "mbox_file", widget);
+    gtk_table_attach_defaults(GTK_TABLE(table), widget, 1, 2, 4, 5);
+    g_object_set_data(G_OBJECT(vbox), "mbox_file", widget);
 
     return vbox;
 }
@@ -751,31 +870,30 @@ static void
 setOptionsCfg(GtkWidget *cfg, ConduitCfg *c)
 {
     GtkWidget *send_action, *from_address, *sendmail, *receive_action, *mbox_file;
-    GtkWidget *menuItem;
     guint id;
 
     /* fetch all the controls from the cfg window */
-    send_action = gtk_object_get_data(GTK_OBJECT(cfg), "send_action");
-    from_address = gtk_object_get_data(GTK_OBJECT(cfg), "from_address");
-    sendmail = gtk_object_get_data(GTK_OBJECT(cfg), "sendmail");
-    receive_action = gtk_object_get_data(GTK_OBJECT(cfg), "receive_action");
-    mbox_file = gtk_object_get_data(GTK_OBJECT(cfg), "mbox_file");
+    send_action = g_object_get_data(G_OBJECT(cfg), "send_action");
+    from_address = g_object_get_data(G_OBJECT(cfg), "from_address");
+    sendmail = g_object_get_data(G_OBJECT(cfg), "sendmail");
+    receive_action = g_object_get_data(G_OBJECT(cfg), "receive_action");
+    mbox_file = g_object_get_data(G_OBJECT(cfg), "mbox_file");
 
-    id = (guint) gtk_object_get_data(GTK_OBJECT(send_action), c->sendAction);
-    gtk_option_menu_set_history(GTK_OPTION_MENU(send_action), id);
+    id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(send_action), c->sendAction));
+    gtk_combo_box_set_active(GTK_COMBO_BOX(send_action), id);
 
     gtk_entry_set_text(GTK_ENTRY(from_address), (c->fromAddr ? c->fromAddr : ""));
     gtk_entry_set_text(GTK_ENTRY(sendmail), (c->sendmail ? c->sendmail : ""));
 
-    id = (guint) gtk_object_get_data(GTK_OBJECT(receive_action), c->receiveAction);
-    gtk_option_menu_set_history(GTK_OPTION_MENU(receive_action), id);
+    id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(receive_action), c->receiveAction));
+    gtk_combo_box_set_active(GTK_COMBO_BOX(receive_action), id);
      
     if (c->mboxFile && 0 != strcmp(c->mboxFile, "")) {
-        gtk_entry_set_text(GTK_ENTRY(mbox_file), c->mboxFile);
+        gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(mbox_file), c->mboxFile);
     } else if (c->mhDirectory) {
-    	gtk_entry_set_text(GTK_ENTRY(mbox_file), c->mhDirectory);
+    	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(mbox_file), c->mhDirectory);
     } else {
-    	gtk_entry_set_text(GTK_ENTRY(mbox_file), "");
+    	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(mbox_file), "");
     }
 }
 
@@ -783,20 +901,19 @@ static void
 readOptionsCfg(GtkWidget *cfg, ConduitCfg *c)
 {
     GtkWidget *send_action, *from_address, *sendmail, *receive_action, *mbox_file;
-    GtkWidget *menu, *menuItem;
     gchar *str;
+    gint id;
     struct stat mboxStat;
 
     /* fetch all the controls from the cfg window */
-    send_action = gtk_object_get_data(GTK_OBJECT(cfg), "send_action");
-    from_address = gtk_object_get_data(GTK_OBJECT(cfg), "from_address");
-    sendmail = gtk_object_get_data(GTK_OBJECT(cfg), "sendmail");
-    receive_action = gtk_object_get_data(GTK_OBJECT(cfg), "receive_action");
-    mbox_file = gtk_object_get_data(GTK_OBJECT(cfg), "mbox_file");
+    send_action = g_object_get_data(G_OBJECT(cfg), "send_action");
+    from_address = g_object_get_data(G_OBJECT(cfg), "from_address");
+    sendmail = g_object_get_data(G_OBJECT(cfg), "sendmail");
+    receive_action = g_object_get_data(G_OBJECT(cfg), "receive_action");
+    mbox_file = g_object_get_data(G_OBJECT(cfg), "mbox_file");
 
-    menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(send_action));
-    menuItem = gtk_menu_get_active(GTK_MENU(menu));
-    str = g_strdup(gtk_object_get_data(GTK_OBJECT(menuItem), "short"));
+    id = gtk_combo_box_get_active(GTK_COMBO_BOX(send_action));
+    str = g_strdup(EC_SEND_ACTION[id]);
     g_free(c->sendAction);
     c->sendAction = str;
 
@@ -816,13 +933,12 @@ readOptionsCfg(GtkWidget *cfg, ConduitCfg *c)
     g_free(c->sendmail);
     c->sendmail = str;
      
-    menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(receive_action));
-    menuItem = gtk_menu_get_active(GTK_MENU(menu));
-    str = g_strdup(gtk_object_get_data(GTK_OBJECT(menuItem), "short"));
+    id = gtk_combo_box_get_active(GTK_COMBO_BOX(receive_action));
+    str = g_strdup(EC_RECV_ACTION[id]);
     g_free(c->receiveAction);
     c->receiveAction = str;
      
-    str = gtk_editable_get_chars(GTK_EDITABLE(mbox_file), 0, -1);
+    str = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(mbox_file));
     if (0 == strcmp(str, "")) {
         g_free(str);
 	str = NULL;
@@ -848,7 +964,7 @@ create_settings_window (GnomePilotConduit *conduit, GtkWidget *parent, gpointer
     gtk_container_add(GTK_CONTAINER(parent),cfgWindow);
     gtk_widget_show_all(cfgWindow);
     
-    gtk_object_set_data(GTK_OBJECT(conduit),OBJ_DATA_CONFIG_WINDOW,cfgWindow);
+    g_object_set_data(G_OBJECT(conduit),OBJ_DATA_CONFIG_WINDOW,cfgWindow);
     setOptionsCfg(GET_CONDUIT_WINDOW(conduit),GET_CONFIG(conduit));
 
     return 0;
@@ -890,23 +1006,23 @@ GnomePilotConduit *conduit_get_gpilot_conduit( guint32 pilotId )
 
   /* conduit signals */
   /*
-  gtk_signal_connect(retval, "copy_from_pilot", (GtkSignalFunc)copy_from_pilot ,NULL);
-  gtk_signal_connect(retval, "copy_to_pilot", (GtkSignalFunc) ,NULL);
-  gtk_signal_connect(retval, "merge_to_pilot", (GtkSignalFunc) ,NULL);
-  gtk_signal_connect(retval, "merge_from_pilot", (GtkSignalFunc)synchronize ,NULL);
+  g_signal_connect(retval, "copy_from_pilot", (GCallback)copy_from_pilot ,NULL);
+  g_signal_connect(retval, "copy_to_pilot", (GCallback) ,NULL);
+  g_signal_connect(retval, "merge_to_pilot", (GCallback) ,NULL);
+  g_signal_connect(retval, "merge_from_pilot", (GCallback)synchronize ,NULL);
   */
-  gtk_signal_connect(retval, "synchronize", (GtkSignalFunc)synchronize ,NULL);
+  g_signal_connect(retval, "synchronize", (GCallback)synchronize ,NULL);
 
   /* GUI signals */
-  gtk_signal_connect(retval, "create_settings_window", (GtkSignalFunc)create_settings_window ,NULL);
-  gtk_signal_connect(retval, "display_settings", (GtkSignalFunc)display_settings ,NULL);
-  gtk_signal_connect(retval, "save_settings", (GtkSignalFunc)save_settings ,NULL);
-  gtk_signal_connect(retval, "revert_settings", (GtkSignalFunc)revert_settings ,NULL);
+  g_signal_connect(retval, "create_settings_window", (GCallback)create_settings_window ,NULL);
+  g_signal_connect(retval, "display_settings", (GCallback)display_settings ,NULL);
+  g_signal_connect(retval, "save_settings", (GCallback)save_settings ,NULL);
+  g_signal_connect(retval, "revert_settings", (GCallback)revert_settings ,NULL);
 
   load_configuration(&cfg1, pilotId );
   cfg2 = dupe_configuration(cfg1);
-  gtk_object_set_data(retval,OBJ_DATA_CONFIG,(gpointer)cfg1);
-  gtk_object_set_data(retval,OBJ_DATA_OLDCONFIG,(gpointer)cfg2);
+  g_object_set_data(G_OBJECT(retval),OBJ_DATA_CONFIG,(gpointer)cfg1);
+  g_object_set_data(G_OBJECT(retval),OBJ_DATA_OLDCONFIG,(gpointer)cfg2);
 
   return GNOME_PILOT_CONDUIT(retval); 
 }
@@ -921,7 +1037,7 @@ void conduit_destroy_gpilot_conduit( GnomePilotConduit *c )
   destroy_configuration( &cfg1 );
   destroy_configuration( &cfg2 );
 
-  inbox_list = (GList*) gtk_object_get_data(GTK_OBJECT(c), "inbox_list");
+  inbox_list = (GList*) g_object_get_data(G_OBJECT(c), "inbox_list");
   for (list = inbox_list; list != NULL; list = list->next) {
     free(list->data);
   }
@@ -931,3 +1047,116 @@ void conduit_destroy_gpilot_conduit( GnomePilotConduit *c )
   gtk_object_destroy(GTK_OBJECT(c));
 }
 
+#define OLD_PREFIX ".gnome2/gnome-pilot.d"
+#define NEW_PREFIX ".gnome-pilot"
+
+#define IS_STR_SET(x) (x != NULL && x[0] != '\0')
+
+static void
+migrate_conf (const gchar *old, const gchar *new)
+{
+	gchar *basename = g_path_get_dirname (new);
+
+	if (!g_file_test (basename, G_FILE_TEST_EXISTS)) {
+		g_mkdir_with_parents (basename, S_IRUSR | S_IWUSR | S_IXUSR);
+	} else {
+		if (!g_file_test (basename, G_FILE_TEST_IS_DIR)) {
+			gchar *tmp = g_strdup_printf ("%s.old", basename);
+			rename (basename, tmp);
+			g_free (tmp);
+			g_mkdir_with_parents (basename, S_IRUSR | S_IWUSR | S_IXUSR);
+		}
+	}
+	g_free (basename);
+
+	if (g_file_test (new, G_FILE_TEST_IS_REGULAR)) {
+		return;
+	} else if (g_file_test (old, G_FILE_TEST_IS_REGULAR)) {
+		rename (old, new);
+	} else {
+		creat (new, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+	}
+}
+
+static GKeyFile*
+get_kfile (const gchar *conf)
+{
+	GKeyFile   *kfile = g_key_file_new ();
+	const char *homedir = g_getenv ("HOME");
+	char       *old = NULL;
+	char       *new = NULL;
+
+	if (!homedir)
+		homedir = g_get_home_dir ();
+
+	old = g_build_filename (homedir, OLD_PREFIX, conf, NULL);
+	new = g_build_filename (homedir, NEW_PREFIX, conf, NULL);
+
+	migrate_conf (old, new);
+
+	g_key_file_load_from_file (kfile, new, G_KEY_FILE_NONE, NULL);
+	g_key_file_set_list_separator (kfile, ' ');
+
+	g_free (new);
+	g_free (old);
+	return kfile;
+}
+
+static gboolean
+save_config (GKeyFile    *kfile,
+	     const gchar *conf)
+{
+	const char *homedir = g_getenv ("HOME");
+        GError     *error = NULL;
+        gchar      *data = NULL;
+        gsize       size;
+	gchar 	   *filename = NULL;
+
+	g_return_val_if_fail (kfile, FALSE);
+	g_return_val_if_fail (IS_STR_SET (conf), FALSE);
+
+	if (!homedir)
+		homedir = g_get_home_dir ();
+
+	filename = g_build_filename (homedir, NEW_PREFIX, conf, NULL);
+
+	if (! g_file_test (filename, G_FILE_TEST_IS_REGULAR)) {
+		g_free (filename);
+		g_warning ("File %s does not exsit", filename);
+		return FALSE;
+	}
+
+        g_message ("Saving config to disk...");
+
+	g_key_file_set_list_separator (kfile, ' ');
+ 	data = g_key_file_to_data (kfile, &size, &error);
+        if (error) {
+                g_warning ("Could not get config data to write to file, %s",
+                           error->message);
+                g_error_free (error);
+
+                return FALSE;
+        }
+
+        g_file_set_contents (filename, data, size, &error);
+        g_free (data);
+
+        if (error) {
+                g_warning ("Could not write %" G_GSIZE_FORMAT " bytes to file '%s', %s",
+                           size,
+                           filename,
+                           error->message);
+                g_free (filename);
+                g_error_free (error);
+
+                return FALSE;
+        }
+
+        g_message ("Wrote config to '%s' (%" G_GSIZE_FORMAT " bytes)",
+                   filename,
+                   size);
+
+	g_free (filename);
+
+	return TRUE;
+}
