@@ -1,4 +1,4 @@
-/* $Id: email_conduit.h 230 2002-09-16 17:16:23Z jpr $ */
+/* $Id$ */
 
 #ifndef __EMAIL_CONDUIT_H__
 #define __EMAIL_CONDUIT_H__
@@ -24,8 +24,8 @@ typedef struct MailDBRecord {
   guchar *buffer;
 } MailDBRecord;
 
-#define GET_CONFIG(c) ((ConduitCfg*)(gtk_object_get_data(GTK_OBJECT(c),OBJ_DATA_CONFIG)))
-#define GET_OLDCONFIG(c) ((ConduitCfg*)(gtk_object_get_data(GTK_OBJECT(c),OBJ_DATA_OLDCONFIG)))
-#define GET_CONDUIT_WINDOW(s) ((GtkWidget*)gtk_object_get_data(GTK_OBJECT(s),OBJ_DATA_CONFIG_WINDOW))
+#define GET_CONFIG(c) ((ConduitCfg*)(g_object_get_data(G_OBJECT(c),OBJ_DATA_CONFIG)))
+#define GET_OLDCONFIG(c) ((ConduitCfg*)(g_object_get_data(G_OBJECT(c),OBJ_DATA_OLDCONFIG)))
+#define GET_CONDUIT_WINDOW(s) ((GtkWidget*)g_object_get_data(G_OBJECT(s),OBJ_DATA_CONFIG_WINDOW))
 
 #endif
