@@ -20,8 +20,14 @@
  *          Vadim Strizhevsky
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <glib.h>
-#include <gnome.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+
 
 #include <pi-source.h>
 #include <pi-socket.h>
@@ -45,6 +51,8 @@
 
 #define MC_DEBUG
 
+#define MC_PAD 8
+
 #ifdef MC_DEBUG
 #define LOG(args...) g_log (G_LOG_DOMAIN, \
                             G_LOG_LEVEL_MESSAGE, \
@@ -55,38 +63,65 @@
 
 GnomePilotConduit *conduit_get_gpilot_conduit (guint32 pilotId);
 void conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit);
+static gboolean save_config (GKeyFile    *kfile, const gchar *conf);
+static GKeyFile* get_kfile (const gchar *conf);
+static void migrate_conf (const gchar *old, const gchar *new);
 
 #define CONDUIT_VERSION "0.9"
 
 static void 
 load_configuration (GnomePilotConduit *conduit, ConduitCfg **c,guint32 pilotId) 
 {
-	char *prefix;
-	char *buf, *key;
+	gchar *iPilot;
+	GKeyFile *kfile;
+	GError *error = NULL;
+
+	char *buf;
+
 	g_return_if_fail (c!=NULL);
 	
-	prefix = g_strdup_printf (CONFIG_PREFIX, pilotId);
- 
+ 	kfile = get_kfile("memo_file-conduit");
+	iPilot = g_strdup_printf ("Pilot_%u", pilotId);
+
 	*c = g_new0 (ConduitCfg, 1);
-	gnome_config_push_prefix (prefix);
+
 	(*c)->sync_type = GnomePilotConduitSyncTypeCustom; /* this will be reset by capplet */
-	(*c)->open_secret = gnome_config_get_bool ("open secret=FALSE"); 
-	
-	buf = gnome_config_get_string ("file mode=0600");
+	(*c)->open_secret = g_key_file_get_boolean (kfile, iPilot, "open secret", &error);
+        if (error) {
+                (*c)->open_secret = FALSE;
+                error = NULL;
+        }
+
+	buf = g_key_file_get_string (kfile, iPilot, "file mode", &error);
+        if (error) {
+                buf = g_strdup("0600");
+                error = NULL;
+        }
 	(*c)->file_mode =(mode_t)strtol (buf, NULL, 0);
 	g_free (buf);
 
-	buf = gnome_config_get_string ("dir mode=0700");
+	buf = g_key_file_get_string (kfile, iPilot, "dir mode", &error);
+        if (error) {
+                buf = g_strdup("0700");
+                error = NULL;
+        }
 	(*c)->dir_mode =(mode_t)strtol (buf, NULL, 0);
 	g_free (buf);
 
-	buf = gnome_config_get_string ("secret mode=0600");
+	buf = g_key_file_get_string (kfile, iPilot, "secret mode", &error);
+        if (error) {
+                buf = g_strdup("0600");
+                error = NULL;
+        }
 	(*c)->secret_mode =(mode_t)strtol (buf, NULL, 0);
 	g_free (buf);
 
-	key = g_strdup_printf ("dir=%s/memo_file", gnome_pilot_conduit_get_base_dir (conduit));
-	(*c)->dir = gnome_config_get_string (key);
-	g_free (key);
+	(*c)->dir = g_key_file_get_string (kfile, iPilot, "dir", &error);
+        if (error) {
+		(*c)->dir = g_strdup_printf ("%s/memo_file",
+		    gnome_pilot_conduit_get_base_dir (conduit));
+                error = NULL;
+	}
 	
 	while ((*c)->dir && strlen ((*c)->dir) > 0 && (*c)->dir[strlen ((*c)->dir)-1] == '/') {
 		(*c)->dir[strlen ((*c)->dir)-1] = '\0';
@@ -100,11 +135,21 @@ load_configuration (GnomePilotConduit *conduit, ConduitCfg **c,guint32 pilotId)
 		}
 	}    
 	
-	(*c)->ignore_end=gnome_config_get_string ("ignore end");
-	(*c)->ignore_start=gnome_config_get_string ("ignore start");
-	gnome_config_pop_prefix ();
-	g_free (prefix);
+	(*c)->ignore_end=g_key_file_get_string (kfile, iPilot, "ignore end", &error);
+        if (error) {
+		(*c)->ignore_end=NULL;
+                error = NULL;
+	}
+
+	(*c)->ignore_start=g_key_file_get_string (kfile, iPilot, "ignore start", &error);
+        if (error) {
+		(*c)->ignore_start=NULL;
+                error = NULL;
+	}
+
 	(*c)->pilotId = pilotId;
+	g_free (iPilot);
+	g_key_file_free (kfile);
 }
 
 static void 
@@ -151,35 +196,36 @@ destroy_configuration (ConduitCfg **c)
 static void 
 save_configuration (ConduitCfg *c) 
 {
-	char *prefix;
+	gchar *iPilot;
+	GKeyFile *kfile;
+
 	char *entry;
 
 	g_return_if_fail (c!=NULL);
-	prefix = g_strdup_printf (CONFIG_PREFIX, c->pilotId);
+ 	kfile = get_kfile("memo_file-conduit");
+	iPilot = g_strdup_printf ("Pilot_%u", c->pilotId);
 
-	gnome_config_push_prefix (prefix);
-	gnome_config_set_bool ("open secret", c->open_secret); 
+	g_key_file_set_boolean (kfile, iPilot, "open secret", c->open_secret);
 
 	entry = g_strdup_printf ("0%o", c->secret_mode);
-	gnome_config_set_string ("secret mode", entry);
+	g_key_file_set_string (kfile, iPilot, "secret mode", entry);
 	g_free (entry);
 
 	entry = g_strdup_printf ("0%o", c->file_mode);
-	gnome_config_set_string ("file mode", entry);
+	g_key_file_set_string (kfile, iPilot, "file mode", entry);
 	g_free (entry);
 
 	entry = g_strdup_printf ("0%o", c->dir_mode);
-	gnome_config_set_string ("dir mode", entry);
+	g_key_file_set_string (kfile, iPilot, "dir mode", entry);
 	g_free (entry);
 
-	gnome_config_set_string ("dir", c->dir);
-	gnome_config_set_string ("ignore end", c->ignore_end);
-	gnome_config_set_string ("ignore start", c->ignore_start);
-	gnome_config_pop_prefix ();
-	g_free (prefix);
+	g_key_file_set_string (kfile, iPilot, "dir", c->dir);
+        g_key_file_set_string (kfile, iPilot, "ignore end", c->ignore_end);
+	g_key_file_set_string (kfile, iPilot, "ignore start", c->ignore_start);
 
-	gnome_config_sync ();
-	gnome_config_drop_all ();
+	g_free (iPilot);
+
+	save_config (kfile, "memo_file-conduit");
 
 }
 
@@ -304,6 +350,8 @@ generate_name (MemoLocalRecord *local, GnomePilotConduitStandardAbs *abs)
 	
 	fname = newfilename (local);
 	categorypath = category_path (local->local.archived?16:local->category, abs);
+	if (local->filename)
+		g_free (local->filename);
 	local->filename = g_strdup_printf ("%s/%s",
 					   categorypath,
 					   fname);
@@ -656,7 +704,7 @@ load_record (GnomePilotConduitStandardAbs *abs, MemoLocalRecord *local)
 	fseek (rec, 0L, SEEK_END);
 	local->length = ftell (rec)+1;
 	rewind (rec);
-	local->record = (unsigned char*)g_malloc (local->length);
+	local->record = (char *)g_malloc (local->length);
 	fread (local->record, local->length-1, 1, rec);
 	local->record[local->length-1]='\0';
 	fclose (rec);
@@ -721,12 +769,6 @@ free_str_foreach (gchar *s, gpointer whatever)
 	g_free (s);
 }
 
-static void
-free_loadinfo_foreach (LoadInfo *info, gpointer whatever) 
-{
-	g_free (info);
-}
-
 /** loads the records into the abs structure */
 static gboolean
 load_records (GnomePilotConduitStandardAbs *abs) 
@@ -762,16 +804,16 @@ load_records (GnomePilotConduitStandardAbs *abs)
 		if (strcmp (de->d_name,".categories")==0) continue;
 		
 		/* first load id:sec:names from the .ids file */
-		loadinfo = g_hash_table_new (g_str_hash, g_str_equal);
+		loadinfo = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
 		filename = g_strdup_printf ("%s/%s", GET_CONDUIT_CFG (abs)->dir, de->d_name);
 		category = GPOINTER_TO_INT (g_hash_table_lookup (categories, filename));
+		g_free (filename);
 		if (category<0 || category > 16) category = 0;
 		if (category == 16) {
 			/* ignore archived dir */
 			continue;
 		}
-		g_free (filename);
 		
 		filename = g_strdup_printf ("%s/%s/.ids", GET_CONDUIT_CFG (abs)->dir, de->d_name);
 		if ((idfile=fopen (filename,"rt"))!=NULL) {
@@ -816,7 +858,6 @@ load_records (GnomePilotConduitStandardAbs *abs)
 						local->local.secret = info->secret;
 						local->mtime=info->mtime;
 						g_hash_table_remove (loadinfo, local->filename);
-						g_free (info);
 					} else {
 						local->local.ID = 0;
 						local->local.secret = 0;
@@ -845,7 +886,6 @@ load_records (GnomePilotConduitStandardAbs *abs)
 		if (g_hash_table_size (loadinfo)>0) {
 			deleted += g_hash_table_size (loadinfo);
 			g_hash_table_foreach (loadinfo,(GHFunc)create_deleted_record_foreach,&GET_CONDUIT_DATA (abs)->records);
-			g_hash_table_foreach (loadinfo,(GHFunc)free_loadinfo_foreach, NULL);
 		}
 		g_hash_table_destroy (loadinfo);
 	}
@@ -868,22 +908,40 @@ static gint
 pre_sync (GnomePilotConduit *c, GnomePilotDBInfo *dbi) 
 {
 	int l;
+#ifdef PILOT_LINK_0_12
+	pi_buffer_t *pi_buf;
+#else
 	unsigned char *buf;
-  
+#endif  
 	g_message ("MemoFile Conduit v %s", CONDUIT_VERSION);
 
 	LOG ("PreSync");
 
 	GET_CONDUIT_DATA (c)->dbi=dbi;
   
+#ifdef PILOT_LINK_0_12
+	pi_buf = pi_buffer_new (0xffff);
+	if ((l=dlp_ReadAppBlock (dbi->pilot_socket, dbi->db_handle, 0,0xffff, pi_buf))<0) {	
+#else
 	buf = (unsigned char*)g_malloc (0xffff);
 	if ((l=dlp_ReadAppBlock (dbi->pilot_socket, dbi->db_handle, 0,(unsigned char *)buf, 0xffff))<0) {
+#endif
 		LOG ("dlp_ReadAppBlock (...) failed");
+#ifdef PILOT_LINK_0_12
+		pi_buffer_free (pi_buf);
+#else
 		g_free (buf);
+#endif
 		return -1;
 	}
+
+#ifdef PILOT_LINK_0_12
+	unpack_MemoAppInfo (&(GET_CONDUIT_DATA (c)->ai), pi_buf->data, l);
+	pi_buffer_free (pi_buf);
+#else
 	unpack_MemoAppInfo (&(GET_CONDUIT_DATA (c)->ai), buf, l);
 	g_free (buf);
+#endif
 
 	if (GET_CONDUIT_CFG (c)->dir==NULL || *(GET_CONDUIT_CFG (c)->dir) == '\0') {
 		return -1;
@@ -991,7 +1049,7 @@ store_remote (GnomePilotConduitStandardAbs *abs,
 			local->record = NULL;
 			local->length = 0;
 		} else {
-			local->record = (unsigned char*)g_malloc (local->length);
+			local->record = (char*)g_malloc (local->length);
 			memcpy (local->record, remote->record, local->length);
 		}
 	}
@@ -1110,7 +1168,7 @@ compare (GnomePilotConduitStandardAbs *abs,
 	g_return_val_if_fail (local!=NULL,-1);
 	g_return_val_if_fail (remote!=NULL,-1);
 	if (local->record==NULL || remote->record == NULL) return -1;
-	return strncmp (local->record, remote->record, local->length);
+	return strncmp (local->record, (char *)remote->record, local->length);
 }
 
 static gint
@@ -1204,13 +1262,13 @@ insert_ignore_space (GtkEditable    *editable, const gchar    *text,
 	curname = gtk_entry_get_text (GTK_ENTRY (editable));
 	if (*curname == '\0' && len > 0) {
 		if (isspace (text[0])) {
-			gtk_signal_emit_stop_by_name (GTK_OBJECT (editable), "insert_text");
+			g_signal_stop_emission_by_name (GTK_OBJECT (editable), "insert_text");
 			return;
 		}
 	} else { 
 		for (i=0; i<len; i++) {
 			if (isspace (text[i])) {
-				gtk_signal_emit_stop_by_name (GTK_OBJECT (editable),"insert_text");
+				g_signal_stop_emission_by_name (GTK_OBJECT (editable),"insert_text");
 				return;
 			}
 		}
@@ -1226,14 +1284,14 @@ insert_numeric_callback (GtkEditable    *editable, const gchar    *text,
 
 	for (i=0; i<len; i++) {
 		if (!isdigit (text[i])) {
-			gtk_signal_emit_stop_by_name (GTK_OBJECT (editable), "insert_text");
+			g_signal_stop_emission_by_name (GTK_OBJECT (editable), "insert_text");
 			return;
 		}
 	}
 }
 
 /* so I like structures. */
-FieldInfo fields[] =
+static FieldInfo fields[] =
 { { N_("Memos directory:"), NULL,"dir", insert_ignore_space},
   { N_("Ignore start:"), NULL,"ignore_start", insert_ignore_space},
   { N_("Ignore end:"), NULL,"ignore_end", insert_ignore_space},
@@ -1250,8 +1308,8 @@ secret_toggled_cb (GtkWidget *widget, gpointer data)
 	ConduitCfg * curState = GET_CONDUIT_CFG (conduit);
 	GtkWidget * main_widget = GET_CONDUIT_WINDOW (conduit);
 	curState->open_secret = GTK_TOGGLE_BUTTON (widget)->active;
-	gtk_widget_set_sensitive (gtk_object_get_data (GTK_OBJECT (main_widget),"secret_mode"), curState->open_secret);
-	gtk_widget_set_sensitive (gtk_object_get_data (GTK_OBJECT (main_widget),"secret_label"), curState->open_secret);
+	gtk_widget_set_sensitive (g_object_get_data (G_OBJECT (main_widget),"secret_mode"), curState->open_secret);
+	gtk_widget_set_sensitive (g_object_get_data (G_OBJECT (main_widget),"secret_label"), curState->open_secret);
 }
 
 
@@ -1266,32 +1324,34 @@ static GtkWidget
 	/* how many fields do we have */
 	while (fields[count].name!=0) count++;
 
-	vbox = gtk_vbox_new (FALSE, GNOME_PAD);
+	vbox = gtk_vbox_new (FALSE, MC_PAD);
 
 	table = gtk_table_new (count, 3, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 10);
-	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, GNOME_PAD);
+	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, MC_PAD);
 
 	for (i=0;i<count;i++) {
 		label = gtk_label_new (_(fields[i].name));
-		gtk_table_attach (GTK_TABLE (table), label, 1, 2, i, i+1, 0, 0, 0, 0);
+		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+                gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, i, i+1);
 		if (fields[i].label_data!=NULL) {
-			gtk_object_set_data (GTK_OBJECT (vbox), fields[i].label_data, label);
+			g_object_set_data (G_OBJECT (vbox), fields[i].label_data, label);
 		}
-		entry = gtk_entry_new_with_max_length (128);
-		gtk_object_set_data (GTK_OBJECT (vbox), fields[i].obj_data, entry);
+		entry = gtk_entry_new();
+		gtk_entry_set_max_length (GTK_ENTRY(entry), 128);
+		g_object_set_data (G_OBJECT (vbox), fields[i].obj_data, entry);
 		gtk_table_attach (GTK_TABLE (table), entry, 2, 3, i, i+1, 0, 0, 0, 0);
-		gtk_signal_connect (GTK_OBJECT (entry), "insert_text",
-				   GTK_SIGNAL_FUNC (fields[i].insert_func),
+		g_signal_connect (GTK_OBJECT (entry), "insert_text",
+				   G_CALLBACK (fields[i].insert_func),
 				   NULL);
 	}
 	
 	button = gtk_check_button_new ();
 
-	gtk_object_set_data (GTK_OBJECT (vbox),"secret_on", button);
-	gtk_signal_connect (GTK_OBJECT (button), "toggled",
-			   GTK_SIGNAL_FUNC (secret_toggled_cb),
+	g_object_set_data (G_OBJECT (vbox),"secret_on", button);
+	g_signal_connect (GTK_OBJECT (button), "toggled",
+			   G_CALLBACK (secret_toggled_cb),
 			   conduit);
 	gtk_table_attach (GTK_TABLE (table), button, 0, 1, 5, 6, 0, 0, 0, 0); /* 5, 6 is badly hard-coded here. */
 
@@ -1307,13 +1367,13 @@ setOptionsCfg (GtkWidget *pilotcfg, ConduitCfg *state)
 	GtkWidget *dir,*ignore_end,*ignore_start,*dir_mode;
 	GtkWidget *secret_button, *file_mode,*secret_mode;
 
-	dir = gtk_object_get_data (GTK_OBJECT (pilotcfg), "dir");
-	ignore_end  = gtk_object_get_data (GTK_OBJECT (pilotcfg), "ignore_end");
-	ignore_start = gtk_object_get_data (GTK_OBJECT (pilotcfg), "ignore_start");
-	dir_mode  = gtk_object_get_data (GTK_OBJECT (pilotcfg), "dir_mode");
-	file_mode = gtk_object_get_data (GTK_OBJECT (pilotcfg), "file_mode");
-	secret_mode = gtk_object_get_data (GTK_OBJECT (pilotcfg), "secret_mode");
-	secret_button = gtk_object_get_data (GTK_OBJECT (pilotcfg),"secret_on");
+	dir = g_object_get_data (G_OBJECT (pilotcfg), "dir");
+	ignore_end  = g_object_get_data (G_OBJECT (pilotcfg), "ignore_end");
+	ignore_start = g_object_get_data (G_OBJECT (pilotcfg), "ignore_start");
+	dir_mode  = g_object_get_data (G_OBJECT (pilotcfg), "dir_mode");
+	file_mode = g_object_get_data (G_OBJECT (pilotcfg), "file_mode");
+	secret_mode = g_object_get_data (G_OBJECT (pilotcfg), "secret_mode");
+	secret_button = g_object_get_data (G_OBJECT (pilotcfg),"secret_on");
 	
 	gtk_entry_set_text (GTK_ENTRY (dir), state->dir);
 	if (state->ignore_start) gtk_entry_set_text (GTK_ENTRY (ignore_start), state->ignore_start);
@@ -1325,8 +1385,8 @@ setOptionsCfg (GtkWidget *pilotcfg, ConduitCfg *state)
 	g_snprintf (buf, 7,"0%o", state->secret_mode);
 	gtk_entry_set_text (GTK_ENTRY (secret_mode), buf);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (secret_button), state->open_secret);
-	gtk_widget_set_sensitive (gtk_object_get_data (GTK_OBJECT (pilotcfg),"secret_mode"), state->open_secret);
-	gtk_widget_set_sensitive (gtk_object_get_data (GTK_OBJECT (pilotcfg),"secret_label"), state->open_secret);
+	gtk_widget_set_sensitive (g_object_get_data (G_OBJECT (pilotcfg),"secret_mode"), state->open_secret);
+	gtk_widget_set_sensitive (g_object_get_data (G_OBJECT (pilotcfg),"secret_label"), state->open_secret);
 
 
 }
@@ -1337,7 +1397,7 @@ readOptionsCfg (GtkWidget *pilotcfg, ConduitCfg *state)
 {
 	GtkWidget *entry; 
 
-	entry  = gtk_object_get_data (GTK_OBJECT (pilotcfg), "dir");
+	entry  = g_object_get_data (G_OBJECT (pilotcfg), "dir");
 	if (state->dir) g_free (state->dir);
 	state->dir = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 
@@ -1345,21 +1405,21 @@ readOptionsCfg (GtkWidget *pilotcfg, ConduitCfg *state)
 		state->dir[strlen (state->dir)-1] = '\0';
 	}
 
-	entry  = gtk_object_get_data (GTK_OBJECT (pilotcfg), "ignore_start");
+	entry  = g_object_get_data (G_OBJECT (pilotcfg), "ignore_start");
 	if (state->ignore_start) g_free (state->ignore_start);
 	state->ignore_start = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 
-	entry  = gtk_object_get_data (GTK_OBJECT (pilotcfg), "ignore_end");
+	entry  = g_object_get_data (G_OBJECT (pilotcfg), "ignore_end");
 	if (state->ignore_end) g_free (state->ignore_end);
 	state->ignore_end = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 
-	entry  = gtk_object_get_data (GTK_OBJECT (pilotcfg), "dir_mode");
+	entry  = g_object_get_data (G_OBJECT (pilotcfg), "dir_mode");
 	state->dir_mode = strtol (gtk_entry_get_text (GTK_ENTRY (entry)), NULL, 0);
 
-	entry  = gtk_object_get_data (GTK_OBJECT (pilotcfg), "file_mode");
+	entry  = g_object_get_data (G_OBJECT (pilotcfg), "file_mode");
 	state->file_mode = strtol (gtk_entry_get_text (GTK_ENTRY (entry)), NULL, 0);
 
-	entry  = gtk_object_get_data (GTK_OBJECT (pilotcfg), "secret_mode");
+	entry  = g_object_get_data (G_OBJECT (pilotcfg), "secret_mode");
 	state->secret_mode = strtol (gtk_entry_get_text (GTK_ENTRY (entry)), NULL, 0);
 }
 
@@ -1374,7 +1434,7 @@ create_settings_window (GnomePilotConduit *conduit, GtkWidget *parent, gpointer
 	gtk_container_add (GTK_CONTAINER (parent), cfgWindow);
 	gtk_widget_show_all (cfgWindow);
 
-	gtk_object_set_data (GTK_OBJECT (conduit), OBJ_DATA_CONFIG_WINDOW, cfgWindow);
+	g_object_set_data (G_OBJECT (conduit), OBJ_DATA_CONFIG_WINDOW, cfgWindow);
 	setOptionsCfg (GET_CONDUIT_WINDOW (conduit), GET_CONDUIT_CFG (conduit));
 	return 0;
 }
@@ -1421,33 +1481,33 @@ conduit_get_gpilot_conduit (guint32 pilotId)
 	LOG ("creating memo_file conduit");
 
 	g_assert (retval != NULL);
-	gtk_signal_connect (retval, "match_record", (GtkSignalFunc) match_record, NULL);
-	gtk_signal_connect (retval, "free_match", (GtkSignalFunc) free_match, NULL);
-	gtk_signal_connect (retval, "archive_local", (GtkSignalFunc) archive_local, NULL);
-	gtk_signal_connect (retval, "archive_remote", (GtkSignalFunc) archive_remote, NULL);
-	gtk_signal_connect (retval, "store_remote", (GtkSignalFunc) store_remote, NULL);
-	gtk_signal_connect (retval, "iterate", (GtkSignalFunc) iterate, NULL);
-	gtk_signal_connect (retval, "iterate_specific", (GtkSignalFunc) iterate_specific, NULL);
-	gtk_signal_connect (retval, "purge", (GtkSignalFunc) purge, NULL);
-	gtk_signal_connect (retval, "set_status", (GtkSignalFunc) set_status, NULL);
-	gtk_signal_connect (retval, "set_pilot_id", (GtkSignalFunc) set_pilot_id, NULL);
-	gtk_signal_connect (retval, "compare", (GtkSignalFunc) compare, NULL);
-	gtk_signal_connect (retval, "compare_backup", (GtkSignalFunc) compare_backup, NULL);
-	gtk_signal_connect (retval, "free_transmit", (GtkSignalFunc) free_transmit, NULL);
-	gtk_signal_connect (retval, "delete_all", (GtkSignalFunc) delete_all, NULL);
-	gtk_signal_connect (retval, "transmit", (GtkSignalFunc) transmit, NULL);
-	gtk_signal_connect (retval, "pre_sync", (GtkSignalFunc) pre_sync, NULL);
+	g_signal_connect (retval, "match_record", (GCallback) match_record, NULL);
+	g_signal_connect (retval, "free_match", (GCallback) free_match, NULL);
+	g_signal_connect (retval, "archive_local", (GCallback) archive_local, NULL);
+	g_signal_connect (retval, "archive_remote", (GCallback) archive_remote, NULL);
+	g_signal_connect (retval, "store_remote", (GCallback) store_remote, NULL);
+	g_signal_connect (retval, "iterate", (GCallback) iterate, NULL);
+	g_signal_connect (retval, "iterate_specific", (GCallback) iterate_specific, NULL);
+	g_signal_connect (retval, "purge", (GCallback) purge, NULL);
+	g_signal_connect (retval, "set_status", (GCallback) set_status, NULL);
+	g_signal_connect (retval, "set_pilot_id", (GCallback) set_pilot_id, NULL);
+	g_signal_connect (retval, "compare", (GCallback) compare, NULL);
+	g_signal_connect (retval, "compare_backup", (GCallback) compare_backup, NULL);
+	g_signal_connect (retval, "free_transmit", (GCallback) free_transmit, NULL);
+	g_signal_connect (retval, "delete_all", (GCallback) delete_all, NULL);
+	g_signal_connect (retval, "transmit", (GCallback) transmit, NULL);
+	g_signal_connect (retval, "pre_sync", (GCallback) pre_sync, NULL);
 
-	gtk_signal_connect (retval, "create_settings_window", (GtkSignalFunc) create_settings_window, NULL);
-	gtk_signal_connect (retval, "display_settings", (GtkSignalFunc) display_settings, NULL);
-	gtk_signal_connect (retval, "save_settings", (GtkSignalFunc) save_settings, NULL);
-	gtk_signal_connect (retval, "revert_settings", (GtkSignalFunc) revert_settings, NULL);
+	g_signal_connect (retval, "create_settings_window", (GCallback) create_settings_window, NULL);
+	g_signal_connect (retval, "display_settings", (GCallback) display_settings, NULL);
+	g_signal_connect (retval, "save_settings", (GCallback) save_settings, NULL);
+	g_signal_connect (retval, "revert_settings", (GCallback) revert_settings, NULL);
 
 	load_configuration (GNOME_PILOT_CONDUIT (retval), &cfg, pilotId);
 	cfg2 = dupe_configuration (cfg);
-	gtk_object_set_data (GTK_OBJECT (retval), OBJ_DATA_CONFIG, cfg);
-	gtk_object_set_data (GTK_OBJECT (retval), OBJ_DATA_OLDCONFIG, cfg2);
-	gtk_object_set_data (GTK_OBJECT (retval), OBJ_DATA_CONDUIT, cd);
+	g_object_set_data (G_OBJECT (retval), OBJ_DATA_CONFIG, cfg);
+	g_object_set_data (G_OBJECT (retval), OBJ_DATA_OLDCONFIG, cfg2);
+	g_object_set_data (G_OBJECT (retval), OBJ_DATA_CONDUIT, cd);
 
 	if (cfg->dir==NULL) {
 		g_warning (_("No dir specified. Please run memo_file conduit capplet first."));
@@ -1484,3 +1544,116 @@ conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit)
 }
 
 
+#define OLD_PREFIX ".gnome2/gnome-pilot.d"
+#define NEW_PREFIX ".gnome-pilot"
+
+#define IS_STR_SET(x) (x != NULL && x[0] != '\0')
+
+static void
+migrate_conf (const gchar *old, const gchar *new)
+{
+	gchar *basename = g_path_get_dirname (new);
+
+	if (!g_file_test (basename, G_FILE_TEST_EXISTS)) {
+		g_mkdir_with_parents (basename, S_IRUSR | S_IWUSR | S_IXUSR);
+	} else {
+		if (!g_file_test (basename, G_FILE_TEST_IS_DIR)) {
+			gchar *tmp = g_strdup_printf ("%s.old", basename);
+			rename (basename, tmp);
+			g_free (tmp);
+			g_mkdir_with_parents (basename, S_IRUSR | S_IWUSR | S_IXUSR);
+		}
+	}
+	g_free (basename);
+
+	if (g_file_test (new, G_FILE_TEST_IS_REGULAR)) {
+		return;
+	} else if (g_file_test (old, G_FILE_TEST_IS_REGULAR)) {
+		rename (old, new);
+	} else {
+		creat (new, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+	}
+}
+
+static GKeyFile*
+get_kfile (const gchar *conf)
+{
+	GKeyFile   *kfile = g_key_file_new ();
+	const char *homedir = g_getenv ("HOME");
+	char       *old = NULL;
+	char       *new = NULL;
+
+	if (!homedir)
+		homedir = g_get_home_dir ();
+
+	old = g_build_filename (homedir, OLD_PREFIX, conf, NULL);
+	new = g_build_filename (homedir, NEW_PREFIX, conf, NULL);
+
+	migrate_conf (old, new);
+
+	g_key_file_load_from_file (kfile, new, G_KEY_FILE_NONE, NULL);
+	g_key_file_set_list_separator (kfile, ' ');
+
+	g_free (new);
+	g_free (old);
+	return kfile;
+}
+
+static gboolean
+save_config (GKeyFile    *kfile,
+	     const gchar *conf)
+{
+	const char *homedir = g_getenv ("HOME");
+        GError     *error = NULL;
+        gchar      *data = NULL;
+        gsize       size;
+	gchar 	   *filename = NULL;
+
+	g_return_val_if_fail (kfile, FALSE);
+	g_return_val_if_fail (IS_STR_SET (conf), FALSE);
+
+	if (!homedir)
+		homedir = g_get_home_dir ();
+
+	filename = g_build_filename (homedir, NEW_PREFIX, conf, NULL);
+
+	if (! g_file_test (filename, G_FILE_TEST_IS_REGULAR)) {
+		g_free (filename);
+		g_warning ("File %s does not exsit", filename);
+		return FALSE;
+	}
+
+        g_message ("Saving config to disk...");
+
+	g_key_file_set_list_separator (kfile, ' ');
+ 	data = g_key_file_to_data (kfile, &size, &error);
+        if (error) {
+                g_warning ("Could not get config data to write to file, %s",
+                           error->message);
+                g_error_free (error);
+
+                return FALSE;
+        }
+
+        g_file_set_contents (filename, data, size, &error);
+        g_free (data);
+
+        if (error) {
+                g_warning ("Could not write %" G_GSIZE_FORMAT " bytes to file '%s', %s",
+                           size,
+                           filename,
+                           error->message);
+                g_free (filename);
+                g_error_free (error);
+
+                return FALSE;
+        }
+
+        g_message ("Wrote config to '%s' (%" G_GSIZE_FORMAT " bytes)",
+                   filename,
+                   size);
+
+	g_free (filename);
+
+	return TRUE;
+}
